package com.persagy.apm.alarmservice.common.enums;

import com.persagy.apm.common.model.annotation.SwaggerDisplayEnum;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang.StringUtils;

/**
 * 报警字典类型
 *
 * @author lixing
 * @version V1.0 2021/9/13 10:01 上午
 **/
@SwaggerDisplayEnum
@AllArgsConstructor
public enum AlarmDicTypeEnum {
    /**
     * 字典类型
     */
    ALARM_CATEGORY("alarm_category", "报警类型"),
    ALARM_LEVEL("alarm_level", "报警等级"),
    ALARM_NATURE("alarm_nature", "报警性质"),
    ALARM_STATE("alarm_state", "报警状态"),
    ALARM_TREAT_MODE("alarm_treat_mode", "报警处理方式"),
    ALARM_TREAT_STATE("alarm_treat_state", "报警处理状态");


    @Setter
    @Getter
    private String type;
    @Setter
    @Getter
    private String desc;

    /**
     * 根据type获取到枚举对象
     *
     * @param type 报告状态编码
     * @return 报告状态枚举对象
     * @author lixing
     * @version V1.0 2021/5/31 4:06 下午
     */
    public static AlarmDicTypeEnum getByType(String type) {
        if (StringUtils.isBlank(type)) {
            return null;
        }
        AlarmDicTypeEnum[] enums = AlarmDicTypeEnum.values();
        for (AlarmDicTypeEnum item : enums) {
            if (type.equals(item.getType())) {
                return item;
            }
        }
        return null;
    }
}


