package com.persagy.apm.alarmservice.dependency.eqdiagnose.client;

import com.persagy.apm.alarmservice.common.constants.FeignClientNameConstants;
import com.persagy.apm.alarmservice.dependency.eqdiagnose.client.fallback.EqDiagnoseClientFallbackFactory;
import com.persagy.apm.alarmservice.dependency.eqdiagnose.model.MonitorIndicatorRecordListItemVO;
import com.persagy.apm.alarmservice.dependency.eqdiagnose.model.QueryMonitorIndicatorRecordDTO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import feign.Headers;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.web.bind.annotation.PostMapping;

/**
 * 设备运行诊断feign客户端
 *
 * @author lixing
 * @version V1.0 2021/3/22 2:13 下午
 **/
@FeignClient(name = FeignClientNameConstants.ENERGY_EQ_RUN_DIAGNOSE, fallbackFactory = EqDiagnoseClientFallbackFactory.class)
public interface EqDiagnoseClient {

    /**
     * 获取监测指标历史采集值
     *
     * @param queryMonitorIndicatorRecordDTO 查询条件
     * @return 监测指标历史采集值列表
     * @author lixing
     * @version V1.0 2021/5/19 3:36 下午
     */
    @PostMapping("/monitorIndicatorRecords/query")
    @Headers({"Content-Type: application/json"})
    CommonResult<PageList<MonitorIndicatorRecordListItemVO>> queryMonitorIndicatorRecord(
            QueryMonitorIndicatorRecordDTO queryMonitorIndicatorRecordDTO);

}
