package com.persagy.apm.alarmservice.group.alarmrule.controller;

import com.persagy.apm.alarmservice.project.alarmcondition.model.AlarmCondition;
import com.persagy.apm.alarmservice.project.alarmcondition.model.ConvertAlarmConditionTool;
import com.persagy.apm.alarmservice.project.alarmcondition.model.vo.AlarmConditionItemVO;
import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.alarmservice.group.alarmrule.dao.GroupAlarmRuleMapper;
import com.persagy.apm.alarmservice.group.alarmrule.model.ConvertGroupAlarmRuleTool;
import com.persagy.apm.alarmservice.group.alarmrule.model.GroupAlarmRule;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.*;
import com.persagy.apm.alarmservice.group.alarmrule.model.vo.GroupAlarmRuleItemVO;
import com.persagy.apm.alarmservice.group.alarmrule.model.vo.GroupAlarmRuleListItemVO;
import com.persagy.apm.alarmservice.group.alarmrule.service.IGroupAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import com.persagy.apm.common.model.vo.SimpleObjVO;
import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.QueryAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmitem.QueryAlarmItemDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmConfigItem;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmConfigServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmItemServiceImpl;
import com.persagy.common.enums.ResponseCode;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 集团报警规则控制层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:35:45
 */
@Api(tags = "集团报警规则")
@Validated
@RestController
@RequestMapping("groupAlarmRules")
public class GroupAlarmRuleController {
    @Autowired
    private IGroupAlarmRuleService groupAlarmRuleService;
    @Autowired
    private GroupAlarmRuleMapper groupAlarmRuleMapper;
    @Autowired
    private IAlarmConditionService alarmConditionService;
    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;

    @Autowired
    AlarmItemServiceImpl alarmItemService;

    @ApiOperation(value = "创建集团报警规则")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createGroupAlarmRule(
            @Valid @RequestBody AddGroupAlarmRuleDTO addGroupAlarmRuleDTO) {
        String id = null;
        try {
            id = groupAlarmRuleService.createGroupAlarmRule(addGroupAlarmRuleDTO);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "集团报警规则详情")
    @PostMapping("/get")
    @ApiIgnore
    public CommonResult<GroupAlarmRuleItemVO> queryGroupAlarmRuleDetail(
            @Valid @RequestBody QueryGroupAlarmRuleDetailDTO queryGroupAlarmRuleDetailDTO) {
        GroupAlarmRule groupAlarmRule = groupAlarmRuleService.queryGroupAlarmRuleDetail(queryGroupAlarmRuleDetailDTO.getId());
        GroupAlarmRuleItemVO groupAlarmRuleItemVO = ConvertGroupAlarmRuleTool.INSTANCE.convert2ResponseItemDTO(groupAlarmRule);
        return ResultHelper.single(groupAlarmRuleItemVO);
    }


    @ApiOperation(value = "更新集团报警规则")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateGroupAlarmRule(
            @Valid @RequestBody UpdateGroupAlarmRuleDTO updateGroupAlarmRuleDTO) {
        try {
            groupAlarmRuleService.updateGroupAlarmRule(updateGroupAlarmRuleDTO);
        } catch (Exception e) {
            e.printStackTrace();
            //return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.single(new ApmResponseUpsertVO(updateGroupAlarmRuleDTO.getId()));
    }


    @ApiOperation(value = "删除集团报警规则")
    @PostMapping("/delete")
    public CommonResult deleteGroupAlarmRule(
            @RequestBody @Valid DeleteGroupAlarmRuleDTO deleteGroupAlarmRuleDTO) {
        try {
            groupAlarmRuleService.delete(deleteGroupAlarmRuleDTO);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.success();
    }


    @ApiOperation(value = "集团报警规则列表")
    @PostMapping("/query")
    public CommonResult<PageList<GroupAlarmRuleListItemVO>> queryGroupAlarmRule(
            @Valid @RequestBody PageQueryGroupAlarmRuleDTO pageQueryGroupAlarmRuleDTO) {
        int page = pageQueryGroupAlarmRuleDTO.getPage();
        int size = pageQueryGroupAlarmRuleDTO.getSize();
        pageQueryGroupAlarmRuleDTO.setStartIndex((page - 1) * size);
        int count = groupAlarmRuleMapper.queryCount(pageQueryGroupAlarmRuleDTO);
        List<GroupAlarmRuleListItemVO> groupAlarmRuleList = groupAlarmRuleMapper.queryList(pageQueryGroupAlarmRuleDTO);
        return ResultHelper.multi(groupAlarmRuleList, count);
    }

//    @ApiOperation(value = "集团报警规则列表, 分页查询")
//    @PostMapping("/query")
//    public CommonResult<PageList<GroupAlarmRuleListItemVO>> queryGroupAlarmRule(
//            @Valid @RequestBody PageQueryGroupAlarmRuleDTO pageQueryGroupAlarmRuleDTO) {
//        IPage<GroupAlarmRule> pagedList = groupAlarmRuleService.pageQueryGroupAlarmRule(pageQueryGroupAlarmRuleDTO);
//        IPage<GroupAlarmRuleListItemVO> pagedResultList = pagedList.convert(item -> {
//            try {
//               return ConvertGroupAlarmRuleTool.INSTANCE.convert2ResponseListItemDTO(item);
//            } catch (Exception e) {
//                throw new IllegalArgumentException("分页查询，GroupAlarmRule实体类转换为列表对象发生异常", e);
//            }
//        });
//        return ResultHelper.multi(pagedResultList);
//    }

    @Autowired
    AlarmConfigServiceImpl alarmConfigService;

    @ApiOperation(value = "")
    @PostMapping("/test")
    @ApiIgnore
    public void test() throws Exception {
        QueryAlarmConfigDTO queryAlarmConfigDTO = new QueryAlarmConfigDTO();
        DmpResult<List<AlarmConfigItem>> query = alarmConfigService.query(queryAlarmConfigDTO);
        System.out.println(query);
    }

    @ApiOperation(value = "读取表格")
    @PostMapping("/readExcel")
    public void readExcel() throws Exception {
        groupAlarmRuleService.readExcel();

    }

    @ApiOperation(value = "集团页面查询报警类型")
    @PostMapping("/getAlarmItem")
    public CommonResult<PageList<SimpleObjVO>> getAlarmItem(@RequestBody CheckAlarmItemDTO checkAlarmItemDTO) throws Exception {
        QueryAlarmItemDTO queryAlarmItemDTO = new QueryAlarmItemDTO();
        queryAlarmItemDTO.setCategory(checkAlarmItemDTO.getCategory());
        DmpResult<List<AlarmItem>> dmpResult = alarmItemService.query(queryAlarmItemDTO);
        List<GroupAlarmRule> existAlarmItem = groupAlarmRuleService.getExistAlarmItem(checkAlarmItemDTO.getGroupId());
        List<String> alarmItemCodes = existAlarmItem.stream().map(GroupAlarmRule::getAlarmItemCode).collect(Collectors.toList());
        Iterator<AlarmItem> iterator = dmpResult.getData().iterator();
        while (iterator.hasNext()) {
            AlarmItem next = iterator.next();
            if (alarmItemCodes.contains(next.getCode())) {
                iterator.remove();
            }
        }
        List<SimpleObjVO> resultList = dmpResult.getData().stream().map(
                alarmItem -> new SimpleObjVO(alarmItem.getCode(), alarmItem.getName())
        ).collect(Collectors.toList());
        return ResultHelper.multi(resultList);
    }

}

