package com.persagy.apm.alarmservice.group.alarmrule.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.alarmservice.group.alarmrule.model.vo.*;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.*;

import java.util.List;

/**
 * 集团报警规则(GroupAlarmRule) dto、vo、do转换工具类
 * @author lixing
 * @version V1.0 2021-09-08 22:35:45
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertGroupAlarmRuleTool {

    ConvertGroupAlarmRuleTool INSTANCE = Mappers.getMapper(ConvertGroupAlarmRuleTool.class);
    
    /**
    * do转换为ResponseItemVO
    *
    * @param groupAlarmRule do对象
    * @return ResponseItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:35:45
    */
    GroupAlarmRuleItemVO convert2ResponseItemDTO(GroupAlarmRule groupAlarmRule);
    
    /**
    * do转换为ResponseListItemVO
    *
    * @param groupAlarmRule do对象
    * @return ResponseListItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:35:45
    */

    GroupAlarmRuleListItemVO convert2ResponseListItemDTO(GroupAlarmRule groupAlarmRule);

    /**
    * do列表转换为ResponseListItemVO列表
    *
    * @param groupAlarmRuleList do列表
    * @return ResponseListItemVO列表
    * @author lixing
    * @version V1.0 2021-09-08 22:35:45
    */
    List<GroupAlarmRuleListItemVO> convert2List(List<GroupAlarmRule> groupAlarmRuleList);
    
    /**
    * addDTO转换为do
    *
    * @param addGroupAlarmRuleDTO addDTO
    * @return 要创建的do对象 
    * @author lixing
    * @version V1.0 2021-09-08 22:35:45
    */
    GroupAlarmRule convertAddDto2Entity(AddGroupAlarmRuleDTO addGroupAlarmRuleDTO);
    
    /**
    * updateDTO转换为实体
    *
    * @param groupAlarmRule 更新前的do对象
    * @param updateGroupAlarmRuleDTO updateDTO
    * @return 更新后的do对象
    * @author lixing
    * @version V1.0 2021-09-08 22:35:45
    */
    GroupAlarmRule convertUpdateDto2Entity(@MappingTarget GroupAlarmRule groupAlarmRule, UpdateGroupAlarmRuleDTO updateGroupAlarmRuleDTO);
}

