package com.persagy.apm.alarmservice.group.calculatemethod.controller;

import com.persagy.apm.alarmservice.group.calculatemethod.model.CalculateMethod;
import com.persagy.apm.alarmservice.group.calculatemethod.model.ConvertCalculateMethodTool;
import com.persagy.apm.alarmservice.group.calculatemethod.model.dto.*;
import com.persagy.apm.alarmservice.group.calculatemethod.model.vo.CalculateMethodItemVO;
import com.persagy.apm.alarmservice.group.calculatemethod.model.vo.CalculateMethodListItemVO;
import com.persagy.apm.alarmservice.group.calculatemethod.service.ICalculateMethodService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.validation.Valid;
import java.util.List;

/**
 * 计算方式控制层
 * @author lixing
 * @version V1.0 2021-09-08 22:43:58
 */
@Api(tags = "计算方式") 
@Validated
@RestController
@RequestMapping("calculateMethods")
public class CalculateMethodController {
    @Autowired
    private ICalculateMethodService calculateMethodService;

	@ApiOperation(value = "创建计算方式")
	@PostMapping("/create")
	public CommonResult<ApmResponseUpsertVO> createCalculateMethod(
	        @Valid @RequestBody AddCalculateMethodDTO addCalculateMethodDTO) {
        String id = calculateMethodService.createCalculateMethod(addCalculateMethodDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
	}
	
	@ApiOperation(value = "计算方式详情")
    @PostMapping("/get")
    public CommonResult<CalculateMethodItemVO> queryCalculateMethodDetail(
            @Valid @RequestBody QueryCalculateMethodDetailDTO queryCalculateMethodDetailDTO) {
        CalculateMethod calculateMethod = calculateMethodService.queryCalculateMethodDetail(queryCalculateMethodDetailDTO.getId());
        CalculateMethodItemVO calculateMethodItemVO = ConvertCalculateMethodTool.INSTANCE.convert2ResponseItemDTO(calculateMethod);
        return ResultHelper.single(calculateMethodItemVO);
    }


    @ApiOperation(value = "更新计算方式")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateCalculateMethod(
            @Valid @RequestBody UpdateCalculateMethodDTO updateCalculateMethodDTO) {
        calculateMethodService.updateCalculateMethod(updateCalculateMethodDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateCalculateMethodDTO.getId()));
    }


    @ApiOperation(value = "删除计算方式")
    @PostMapping("/delete")
    public CommonResult deleteCalculateMethod(
            @RequestBody @Valid DeleteCalculateMethodDTO deleteCalculateMethodDTO) {
        calculateMethodService.deleteCalculateMethod(deleteCalculateMethodDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "计算方式列表")
    @PostMapping("/query")
    public CommonResult<PageList<CalculateMethodListItemVO>> queryCalculateMethod(
            @Valid @RequestBody QueryCalculateMethodDTO queryCalculateMethodDTO) {
        List<CalculateMethod> calculateMethodList = calculateMethodService.queryCalculateMethodList(queryCalculateMethodDTO);
        List<CalculateMethodListItemVO> resultList = ConvertCalculateMethodTool.INSTANCE.convert2List(calculateMethodList);
        return ResultHelper.multi(resultList);
    }
    
    //@ApiOperation(value = "计算方式列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<CalculateMethodListItemVO>> queryCalculateMethod(
    //        @Valid @RequestBody PageQueryCalculateMethodDTO pageQueryCalculateMethodDTO) {
    //    IPage<CalculateMethod> pagedList = calculateMethodService.pageQueryCalculateMethod(pageQueryCalculateMethodDTO);
    //    IPage<CalculateMethodListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertCalculateMethodTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，CalculateMethod实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

