package com.persagy.apm.alarmservice.group.calculatemethod.model;

import com.persagy.apm.alarmservice.group.calculatemethod.model.dto.AddCalculateMethodDTO;
import com.persagy.apm.alarmservice.group.calculatemethod.model.dto.UpdateCalculateMethodDTO;
import com.persagy.apm.alarmservice.group.calculatemethod.model.vo.CalculateMethodItemVO;
import com.persagy.apm.alarmservice.group.calculatemethod.model.vo.CalculateMethodListItemVO;
import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.List;

/**
 * 计算方式(CalculateMethod) dto、vo、do转换工具类
 * @author lixing
 * @version V1.0 2021-09-08 22:43:58
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertCalculateMethodTool {

    ConvertCalculateMethodTool INSTANCE = Mappers.getMapper(ConvertCalculateMethodTool.class);
    
    /**
    * do转换为ResponseItemVO
    *
    * @param calculateMethod do对象
    * @return ResponseItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    CalculateMethodItemVO convert2ResponseItemDTO(CalculateMethod calculateMethod);
    
    /**
    * do转换为ResponseListItemVO
    *
    * @param calculateMethod do对象
    * @return ResponseListItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    CalculateMethodListItemVO convert2ResponseListItemDTO(CalculateMethod calculateMethod);

    /**
    * do列表转换为ResponseListItemVO列表
    *
    * @param calculateMethodList do列表
    * @return ResponseListItemVO列表
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    List<CalculateMethodListItemVO> convert2List(List<CalculateMethod> calculateMethodList);
    
    /**
    * addDTO转换为do
    *
    * @param addCalculateMethodDTO addDTO
    * @return 要创建的do对象 
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    CalculateMethod convertAddDto2Entity(AddCalculateMethodDTO addCalculateMethodDTO);
    
    /**
    * updateDTO转换为实体
    *
    * @param calculateMethod 更新前的do对象
    * @param updateCalculateMethodDTO updateDTO
    * @return 更新后的do对象
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    CalculateMethod convertUpdateDto2Entity(@MappingTarget CalculateMethod calculateMethod, UpdateCalculateMethodDTO updateCalculateMethodDTO);
}

