package com.persagy.apm.alarmservice.group.monitorindicator.controller;

import com.persagy.apm.alarmservice.group.monitorindicator.model.ConvertMonitorIndicatorTool;
import com.persagy.apm.alarmservice.group.monitorindicator.model.MonitorIndicator;
import com.persagy.apm.alarmservice.group.monitorindicator.model.dto.*;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorItemVO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorListItemVO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorRecordInfoItem;
import com.persagy.apm.alarmservice.group.monitorindicator.service.IMonitorIndicatorService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * 监测指标控制层
 * @author lixing
 * @version V1.0 2021-09-08 22:44:37
 */
@Api(tags = "监测指标") 
@Validated
@RestController
@RequestMapping("monitorIndicators")
public class MonitorIndicatorController {
    @Autowired
    private IMonitorIndicatorService monitorIndicatorService;

	@ApiOperation(value = "创建监测指标")
	@PostMapping("/create")
    @ApiIgnore
	public CommonResult<ApmResponseUpsertVO> createMonitorIndicator(
	        @Valid @RequestBody AddMonitorIndicatorDTO addMonitorIndicatorDTO) {
        String id = monitorIndicatorService.createMonitorIndicator(addMonitorIndicatorDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
	}
	
	@ApiOperation(value = "监测指标详情")
    @PostMapping("/get")
    public CommonResult<MonitorIndicatorItemVO> queryMonitorIndicatorDetail(
            @Valid @RequestBody QueryMonitorIndicatorDetailDTO queryMonitorIndicatorDetailDTO) {
        MonitorIndicator monitorIndicator = monitorIndicatorService.queryMonitorIndicatorDetail(queryMonitorIndicatorDetailDTO.getId());
        MonitorIndicatorItemVO monitorIndicatorItemVO = ConvertMonitorIndicatorTool.INSTANCE.convert2ResponseItemDTO(monitorIndicator);
        return ResultHelper.single(monitorIndicatorItemVO);
    }


    @ApiOperation(value = "更新监测指标")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateMonitorIndicator(
            @Valid @RequestBody UpdateMonitorIndicatorDTO updateMonitorIndicatorDTO) {
        monitorIndicatorService.updateMonitorIndicator(updateMonitorIndicatorDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateMonitorIndicatorDTO.getId()));
    }


    @ApiOperation(value = "删除监测指标")
    @PostMapping("/delete")
    public CommonResult deleteMonitorIndicator(
            @RequestBody @Valid DeleteMonitorIndicatorDTO deleteMonitorIndicatorDTO) {
        monitorIndicatorService.deleteMonitorIndicator(deleteMonitorIndicatorDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "监测指标列表")
    @PostMapping("/query")
    public CommonResult<PageList<MonitorIndicatorListItemVO>> queryMonitorIndicator(
            @Valid @RequestBody QueryMonitorIndicatorDTO queryMonitorIndicatorDTO) {
        List<MonitorIndicator> monitorIndicatorList = monitorIndicatorService.queryMonitorIndicatorList(queryMonitorIndicatorDTO);
        List<MonitorIndicatorListItemVO> resultList = ConvertMonitorIndicatorTool.INSTANCE.convert2List(monitorIndicatorList);
        monitorIndicatorService.setIndicatorMethodCount(resultList);
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "监测指标数据信息")
    @PostMapping("/record_info")
    public CommonResult<PageList<MonitorIndicatorRecordInfoItem>> monitorIndicatorRecordInfo(
            @Valid @RequestBody QueryMonitorIndicatorRecordInfoDTO queryMonitorIndicatorRecordInfoDTO) throws Exception{
        List<MonitorIndicatorRecordInfoItem> list = monitorIndicatorService.monitorIndicatorRecordInfo(queryMonitorIndicatorRecordInfoDTO);
        return ResultHelper.multi(list);
    }
}

