package com.persagy.apm.alarmservice.group.monitorindicator.model;

import com.persagy.apm.alarmservice.group.monitorindicator.model.dto.AddMonitorIndicatorDTO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.dto.UpdateMonitorIndicatorDTO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorItemVO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorListItemVO;
import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.List;

/**
 * 监测指标(MonitorIndicator) dto、vo、do转换工具类
 * @author lixing
 * @version V1.0 2021-09-08 22:44:38
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertMonitorIndicatorTool {

    ConvertMonitorIndicatorTool INSTANCE = Mappers.getMapper(ConvertMonitorIndicatorTool.class);
    
    /**
    * do转换为ResponseItemVO
    *
    * @param monitorIndicator do对象
    * @return ResponseItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:44:38
    */
    MonitorIndicatorItemVO convert2ResponseItemDTO(MonitorIndicator monitorIndicator);
    
    /**
    * do转换为ResponseListItemVO
    *
    * @param monitorIndicator do对象
    * @return ResponseListItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:44:38
    */
    MonitorIndicatorListItemVO convert2ResponseListItemDTO(MonitorIndicator monitorIndicator);

    /**
    * do列表转换为ResponseListItemVO列表
    *
    * @param monitorIndicatorList do列表
    * @return ResponseListItemVO列表
    * @author lixing
    * @version V1.0 2021-09-08 22:44:38
    */
    List<MonitorIndicatorListItemVO> convert2List(List<MonitorIndicator> monitorIndicatorList);
    
    /**
    * addDTO转换为do
    *
    * @param addMonitorIndicatorDTO addDTO
    * @return 要创建的do对象 
    * @author lixing
    * @version V1.0 2021-09-08 22:44:38
    */
    MonitorIndicator convertAddDto2Entity(AddMonitorIndicatorDTO addMonitorIndicatorDTO);
    
    /**
    * updateDTO转换为实体
    *
    * @param monitorIndicator 更新前的do对象
    * @param updateMonitorIndicatorDTO updateDTO
    * @return 更新后的do对象
    * @author lixing
    * @version V1.0 2021-09-08 22:44:38
    */
    MonitorIndicator convertUpdateDto2Entity(@MappingTarget MonitorIndicator monitorIndicator, UpdateMonitorIndicatorDTO updateMonitorIndicatorDTO);
}

