package com.persagy.apm.alarmservice.group.monitorindicator.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.apm.alarmservice.group.monitorindicator.model.MonitorIndicator;
import com.persagy.apm.alarmservice.group.monitorindicator.model.dto.*;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorListItemVO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorRecordInfoItem;

import java.util.List;

/**
 * 监测指标(MonitorIndicator) service接口
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:44:37
 */
public interface IMonitorIndicatorService {

    /**
     * 创建监测指标
     *
     * @param addMonitorIndicatorDTO addDTO
     * @return 监测指标主键
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public String createMonitorIndicator(AddMonitorIndicatorDTO addMonitorIndicatorDTO);

    /**
     * 监测指标详情
     *
     * @param id 主键
     * @return 监测指标do对象
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public MonitorIndicator queryMonitorIndicatorDetail(String id);

    /**
     * 更新监测指标
     *
     * @param updateMonitorIndicatorDTO updateDTO
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public void updateMonitorIndicator(UpdateMonitorIndicatorDTO updateMonitorIndicatorDTO);

    /**
     * 删除监测指标
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public void deleteMonitorIndicator(String id);

    /**
     * 查询监测指标
     *
     * @param queryMonitorIndicatorDTO queryDTO
     * @return List<MonitorIndicator>
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public List<MonitorIndicator> queryMonitorIndicatorList(QueryMonitorIndicatorDTO queryMonitorIndicatorDTO);

    /**
     * 分页查询监测指标
     *
     * @param pageQueryMonitorIndicatorDTO pageQueryDTO
     * @return IPage<MonitorIndicator>
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public IPage<MonitorIndicator> pageQueryMonitorIndicator(PageQueryMonitorIndicatorDTO pageQueryMonitorIndicatorDTO);

    /**
     * 为监测指标包含的计算方式数量赋值
     *
     * @param indicators 监测指标列表
     * @return 计算方式数量赋值后的监测指标列表
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public void setIndicatorMethodCount(List<MonitorIndicatorListItemVO> indicators);

    /**
     * 查询报警记录相关的监测指标取值
     *
     * @param queryMonitorIndicatorRecordInfoDTO 查询条件
     * @return 报警记录相关的监测指标取值
     * @author lixing
     * @version V1.0 2021/9/27 10:47 上午
     */
    List<MonitorIndicatorRecordInfoItem> monitorIndicatorRecordInfo(
            QueryMonitorIndicatorRecordInfoDTO queryMonitorIndicatorRecordInfoDTO) throws Exception;
}
