package com.persagy.apm.alarmservice.group.monitorindicator.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.googlecode.aviator.AviatorEvaluator;
import com.googlecode.aviator.Expression;
import com.persagy.apm.alarmservice.dependency.eqdiagnose.model.QueryMonitorIndicatorRecordDTO;
import com.persagy.apm.alarmservice.dependency.eqdiagnose.service.EqDiagnoseServiceImpl;
import com.persagy.apm.alarmservice.group.alarmrecord.service.AlarmRecordWrapper;
import com.persagy.apm.alarmservice.group.calculatemethod.model.CalculateMethod;
import com.persagy.apm.alarmservice.group.calculatemethod.model.dto.QueryCalculateMethodDTO;
import com.persagy.apm.alarmservice.group.calculatemethod.service.ICalculateMethodService;
import com.persagy.apm.alarmservice.group.monitorindicator.dao.MonitorIndicatorMapper;
import com.persagy.apm.alarmservice.group.monitorindicator.model.ConvertMonitorIndicatorTool;
import com.persagy.apm.alarmservice.group.monitorindicator.model.MonitorIndicator;
import com.persagy.apm.alarmservice.group.monitorindicator.model.dto.*;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorFormulaItem;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorListItemVO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorRecordInfoItem;
import com.persagy.apm.alarmservice.group.monitorindicator.model.vo.MonitorIndicatorReferenceItem;
import com.persagy.apm.alarmservice.group.monitorindicator.service.IMonitorIndicatorService;
import com.persagy.apm.common.constant.enums.AttrValueTypeEnum;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.common.model.vo.GraphItemVO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmRecordItem;
import org.apache.commons.lang.StringUtils;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * 监测指标(MonitorIndicator) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:44:37
 */
@Service
public class MonitorIndicatorServiceImpl extends ServiceImpl<MonitorIndicatorMapper, MonitorIndicator>
        implements IMonitorIndicatorService {
    @Autowired
    ICalculateMethodService calculateMethodService;
    @Autowired
    AlarmRecordWrapper alarmRecordWrapper;
    @Autowired
    EqDiagnoseServiceImpl eqDiagnoseService;
    @Autowired
    MonitorIndicatorServiceImpl monitorIndicatorService;

    /**
     * 创建监测指标
     *
     * @return 监测指标主键
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public String createMonitorIndicator(AddMonitorIndicatorDTO addMonitorIndicatorDTO) {
        MonitorIndicator monitorIndicator = ConvertMonitorIndicatorTool.INSTANCE.convertAddDto2Entity(addMonitorIndicatorDTO);
        // 设置默认值
        setDefaultValue(monitorIndicator);
        save(monitorIndicator);
        return monitorIndicator.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param monitorIndicator 监测指标实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(MonitorIndicator monitorIndicator) {
        monitorIndicator.setCreator(AppContext.getContext().getAccountId());
    }

    /**
     * 监测指标详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public MonitorIndicator queryMonitorIndicatorDetail(String id) {
        MonitorIndicator monitorIndicator = getById(id);
        if (monitorIndicator == null) {
            throw new IllegalArgumentException("查看MonitorIndicator详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return monitorIndicator;
    }

    /**
     * 更新监测指标
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public void updateMonitorIndicator(UpdateMonitorIndicatorDTO updateMonitorIndicatorDTO) {
        MonitorIndicator monitorIndicator = getById(updateMonitorIndicatorDTO.getId());
        monitorIndicator = ConvertMonitorIndicatorTool.INSTANCE.convertUpdateDto2Entity(monitorIndicator, updateMonitorIndicatorDTO);
        monitorIndicator.setModifier(AppContext.getContext().getAccountId());
        updateById(monitorIndicator);
    }

    /**
     * 校验监测指标是否可删除
     *
     * @param id 监测指标主键
     * @return 监测指标do类
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public MonitorIndicator checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除MonitorIndicator时发生异常，主键为空");
        }

        MonitorIndicator monitorIndicator = getById(id);

        if (monitorIndicator == null) {
            throw new IllegalArgumentException("删除MonitorIndicator时发生异常，找不到要删除的数据，id:" + id);
        }

        return monitorIndicator;
    }

    /**
     * 删除监测指标
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public void deleteMonitorIndicator(String id) {
        // 校验是否可删除
        MonitorIndicator monitorIndicator = checkDeletable(id);

        monitorIndicator.setValid(ValidEnum.FALSE.getType());
        updateById(monitorIndicator);
    }

    /**
     * 查询监测指标
     *
     * @return List<MonitorIndicator>
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public List<MonitorIndicator> queryMonitorIndicatorList(QueryMonitorIndicatorDTO queryMonitorIndicatorDTO) {
        QueryWrapper<MonitorIndicator> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(MonitorIndicator.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, MonitorIndicator.PROP_CREATIONTIME);

        if (queryMonitorIndicatorDTO != null) {

            if (queryMonitorIndicatorDTO.getIds() != null) {
                if (CollectionUtils.isEmpty(queryMonitorIndicatorDTO.getIds())) {
                    return new ArrayList<>();
                }
                queryWrapper.in(MonitorIndicator.PROP_ID, queryMonitorIndicatorDTO.getIds());
            }


            if (StringUtils.isNotEmpty(queryMonitorIndicatorDTO.getName())) {
                queryWrapper.like(MonitorIndicator.PROP_NAME, queryMonitorIndicatorDTO.getName());
            }

            if (StringUtils.isNotEmpty(queryMonitorIndicatorDTO.getClassCode())) {
                queryWrapper.eq(MonitorIndicator.PROP_CLASS_CODE, queryMonitorIndicatorDTO.getClassCode());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询监测指标
     *
     * @return IPage<MonitorIndicator>
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public IPage<MonitorIndicator> pageQueryMonitorIndicator(PageQueryMonitorIndicatorDTO pageQueryMonitorIndicatorDTO) {
        QueryWrapper<MonitorIndicator> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(MonitorIndicator.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<MonitorIndicator> pageParam = new Page<>(pageQueryMonitorIndicatorDTO.getPage(), pageQueryMonitorIndicatorDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryMonitorIndicatorDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, MonitorIndicator.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryMonitorIndicatorDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryMonitorIndicatorDTO.getName())) {
            queryWrapper.like(MonitorIndicator.PROP_NAME, pageQueryMonitorIndicatorDTO.getName());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryMonitorIndicatorDTO.getClassCode())) {
            queryWrapper.like(MonitorIndicator.PROP_CLASS_CODE, pageQueryMonitorIndicatorDTO.getClassCode());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public void setIndicatorMethodCount(List<MonitorIndicatorListItemVO> indicators) {
        indicators.forEach(indicator -> {
            QueryCalculateMethodDTO queryCalculateMethodDTO = new QueryCalculateMethodDTO();
            queryCalculateMethodDTO.setMonitorIndicatorId(indicator.getId());
            List<CalculateMethod> calculateMethods = calculateMethodService.
                    queryCalculateMethodList(queryCalculateMethodDTO);
            int methodCount = calculateMethods == null ? 0 : calculateMethods.size();
            indicator.setMethodCount(methodCount);
        });
    }

    @Override
    public List<MonitorIndicatorRecordInfoItem> monitorIndicatorRecordInfo(
            QueryMonitorIndicatorRecordInfoDTO queryMonitorIndicatorRecordInfoDTO) throws Exception {
        // 获取报警记录关联的指标和指标关联的公式
        String alarmRecordId = queryMonitorIndicatorRecordInfoDTO.getAlarmRecordId();
        AlarmRecordItem alarmRecord = alarmRecordWrapper.getAlarmRecord(alarmRecordId);
        JSONObject condition = alarmRecord.getCondition();
        JSONArray infoCodeArray = condition.getJSONArray("infoCode");
        JSONArray conditionArray = condition.getJSONArray("configs");
        if (CollectionUtils.isEmpty(infoCodeArray) || CollectionUtils.isEmpty(conditionArray)) {
            return new ArrayList<>();
        }

        // 获取监测指标的历史采集值 monitorId -> (time -> value)
        List<String> infoCodeList = infoCodeArray.toJavaList(String.class);
        Map<String, Map<Date, String>> monitorValuesMap = getIndicatorHistoryValues(
                alarmRecord.getObjId(),
                infoCodeList, queryMonitorIndicatorRecordInfoDTO.getStartTime(),
                queryMonitorIndicatorRecordInfoDTO.getEndTime());

        // indicatorId -> list<formula> 监测指标和与监测指标关联的公式的映射
        Map<String, List<MonitorIndicatorFormulaItem>> indicatorFormulaMap = getIndicatorFormulaMap(conditionArray);

        // 遍历监测指标和与监测指标关联的公式的映射，获取监测指标和公式的取值，转换为返回结果
        List<MonitorIndicatorRecordInfoItem> result = new ArrayList<>();
        for (String indicatorId : indicatorFormulaMap.keySet()) {
            MonitorIndicatorRecordInfoItem monitorIndicatorRecordInfoItem = new MonitorIndicatorRecordInfoItem();
            // 查询监测指标基本信息
            MonitorIndicator monitorIndicator = monitorIndicatorService.queryMonitorIndicatorDetail(indicatorId);
            monitorIndicatorRecordInfoItem.setMonitorIndicatorName(monitorIndicator.getName());
            monitorIndicatorRecordInfoItem.setUnit(monitorIndicator.getUnit());
            // 获取监测指标历史采集值
            Map<Date, String> indicatorRecordMap = monitorValuesMap.get(indicatorId);
            List<GraphItemVO> values = getIndicatorRecordGraphData(monitorIndicator, indicatorRecordMap);
            monitorIndicatorRecordInfoItem.setValue(values);
            // 如果监测指标的取值不是枚举数据，获取监测指标参照值
            if (monitorIndicator.getDic() != null) {
                monitorIndicatorRecordInfoItem.setDic(monitorIndicator.getDic());
            } else {
                List<MonitorIndicatorReferenceItem> reference = getMonitorIndicatorReferenceItems(
                        monitorValuesMap, indicatorFormulaMap, monitorIndicator);
                monitorIndicatorRecordInfoItem.setReference(reference);
            }
            result.add(monitorIndicatorRecordInfoItem);
        }
        return result;
    }

    /**
     * 获取监测指标的参照值列表
     *
     * @param monitorValuesMap    所有相关监测指标的历史采集值 indicatorId -> (date -> value)
     * @param indicatorFormulaMap 所有监测指标关联的公式 indicatorId -> 公式列表
     * @param monitorIndicator    监测指标对象
     * @return 监测指标的参照值列表
     * @author lixing
     * @version V1.0 2021/9/27 4:23 下午
     */
    private List<MonitorIndicatorReferenceItem> getMonitorIndicatorReferenceItems(
            Map<String, Map<Date, String>> monitorValuesMap,
            Map<String, List<MonitorIndicatorFormulaItem>> indicatorFormulaMap, MonitorIndicator monitorIndicator) {
        List<MonitorIndicatorReferenceItem> reference = new ArrayList<>();
        List<MonitorIndicatorFormulaItem> monitorIndicatorFormulaItems = indicatorFormulaMap.get(monitorIndicator.getId());
        for (MonitorIndicatorFormulaItem monitorIndicatorFormulaItem : monitorIndicatorFormulaItems) {
            MonitorIndicatorReferenceItem monitorIndicatorReferenceItem = new MonitorIndicatorReferenceItem();
            monitorIndicatorReferenceItem.setUnit(monitorIndicator.getUnit());
            monitorIndicatorReferenceItem.setOperator(monitorIndicatorFormulaItem.getOperator());
            // 解析公式，得到公式计算结果
            String formula = monitorIndicatorFormulaItem.getFormula();
            List<GraphItemVO> referenceValueList = new ArrayList<>();
            Map<Date, String> indicatorRecordMap = monitorValuesMap.get(monitorIndicator.getId());
            if (!CollectionUtils.isEmpty(indicatorRecordMap)) {
                for (Date date : indicatorRecordMap.keySet()) {
                    HashMap<String, Object> paramMap = getParamMap(monitorValuesMap, date);
                    Double formulaResult = calculateFormula(formula, paramMap);
                    GraphItemVO graphItemVO = new GraphItemVO(
                            String.valueOf(date.getTime()), formulaResult,
                            AttrValueTypeEnum.number_2, monitorIndicator.getUnit());
                    referenceValueList.add(graphItemVO);
                }
            }

            monitorIndicatorReferenceItem.setValue(referenceValueList);
            reference.add(monitorIndicatorReferenceItem);
        }
        return reference;
    }

    /**
     * 获取监测指标历史采集值图表数据
     *
     * @param monitorIndicator   监测指标对象
     * @param indicatorRecordMap 监测指标历史采集值 date->value
     * @return 监测指标历史采集值图表数据
     * @author lixing
     * @version V1.0 2021/9/27 4:20 下午
     */
    private List<GraphItemVO> getIndicatorRecordGraphData(
            MonitorIndicator monitorIndicator, Map<Date, String> indicatorRecordMap) {
        List<GraphItemVO> values = new ArrayList<>();
        if (CollectionUtils.isEmpty(indicatorRecordMap)) {
            return values;
        }
        for (Date date : indicatorRecordMap.keySet()) {
            if (date == null) {
                continue;
            }
            String valueStr = indicatorRecordMap.get(date);
            Double value = StringUtils.isBlank(valueStr) ? null : Double.valueOf(valueStr);
            GraphItemVO graphItemVO = new GraphItemVO(
                    String.valueOf(date.getTime()), value, AttrValueTypeEnum.number, monitorIndicator.getUnit());
            values.add(graphItemVO);
        }
        return values;
    }

    /**
     * 将给定时间各个指标的取值转换为公式参数map
     *
     * @param monitorValuesMap 监测指标的历史采集值 monitorId -> (time -> value)
     * @param date             给定时间
     * @return 公式参数map
     * @author lixing
     * @version V1.0 2021/9/27 4:00 下午
     */
    private HashMap<String, Object> getParamMap(Map<String, Map<Date, String>> monitorValuesMap, Date date) {
        if (CollectionUtils.isEmpty(monitorValuesMap)) {
            return new HashMap<>();
        }

        HashMap<String, Object> paramMap = new HashMap<>();
        for (String indicatorId : monitorValuesMap.keySet()) {
            Map<Date, String> recordMap = monitorValuesMap.get(indicatorId);
            if (CollectionUtils.isEmpty(recordMap)) {
                continue;
            }
            if (recordMap.get(date) != null) {
                paramMap.put(indicatorId, Double.valueOf(recordMap.get(date)));
            }
        }
        return paramMap;
    }

    /**
     * 计算公式，获取计算结果
     *
     * @param formula  公式
     * @param paramMap 参数
     * @return 结果
     * @author lixing
     * @version V1.0 2021/9/27 3:59 下午
     */
    private Double calculateFormula(String formula, HashMap<String, Object> paramMap) {
        Expression triggerExp = AviatorEvaluator.compile(formula, true);
        Object resultObj = triggerExp.execute(paramMap);
        if (resultObj == null) {
            return null;
        }
        return Double.valueOf(resultObj.toString());
    }

    /**
     * 获取监测指标和与监测指标关联的公式的映射
     *
     * @param conditionArray 报警条件数组
     * @return 监测指标和与监测指标关联的公式的映射
     * @author lixing
     * @version V1.0 2021/9/27 2:42 下午
     */
    private Map<String, List<MonitorIndicatorFormulaItem>> getIndicatorFormulaMap(JSONArray conditionArray) {
        Map<String, List<MonitorIndicatorFormulaItem>> indicatorFormulaMap = new HashMap<>();
        // conditionArray: [condition], condition: [rule], rule: indicator, formula, operator
        for (Object conditionObj : conditionArray) {
            JSONArray ruleArray = (JSONArray) conditionObj;
            if (CollectionUtils.isEmpty(ruleArray)) {
                continue;
            }
            for (Object ruleObj : ruleArray) {
                JSONObject rule = (JSONObject) ruleObj;
                String monitorIndicatorId = rule.getString("monitorIndicatorId");
                if (StringUtils.isBlank(monitorIndicatorId)) {
                    continue;
                }
                String tempFormula = rule.getString("tempFormula");
                String operator = rule.getString("operator");
                if (indicatorFormulaMap.containsKey(monitorIndicatorId)) {
                    List<MonitorIndicatorFormulaItem> monitorIndicatorFormulaItems = indicatorFormulaMap.get(monitorIndicatorId);
                    monitorIndicatorFormulaItems.add(new MonitorIndicatorFormulaItem(operator, tempFormula));
                } else {
                    indicatorFormulaMap.put(monitorIndicatorId,
                            Lists.newArrayList(new MonitorIndicatorFormulaItem(operator, tempFormula)));
                }
            }
        }
        return indicatorFormulaMap;
    }

    /**
     * 获取监测指标的历史采集值
     *
     * @param objId        设备id
     * @param infoCodeList [equipId-monitorId]
     * @param startTime    开始时间
     * @param endTime      结束时间
     * @return 监测指标的历史采集值 monitorId -> (time -> value)
     * @author lixing
     * @version V1.0 2021/9/27 2:35 下午
     */
    private Map<String, Map<Date, String>> getIndicatorHistoryValues(
            String objId, List<String> infoCodeList, Date startTime, Date endTime) {
        // monitorId -> (time -> value)
        Map<String, Map<Date, String>> monitorValuesMap = new HashMap<>();
        // infoCodeList：[indicatorId]
        for (String indicatorId : infoCodeList) {
            // 查询设备监测指标取值
            QueryMonitorIndicatorRecordDTO queryMonitorIndicatorRecordDTO = new QueryMonitorIndicatorRecordDTO();
            queryMonitorIndicatorRecordDTO.setObjId(objId);
            queryMonitorIndicatorRecordDTO.setProjectId(PoemsContext.getContext().getProjectId());
            queryMonitorIndicatorRecordDTO.setMonitorIndicatorId(indicatorId);
            queryMonitorIndicatorRecordDTO.setTimeFrom(startTime == null ? null : startTime.getTime());
            queryMonitorIndicatorRecordDTO.setTimeTo(endTime == null ? null : endTime.getTime());
            Map<Date, String> indicatorRecordMap = eqDiagnoseService.
                    queryMonitorIndicatorRecord(queryMonitorIndicatorRecordDTO);
            monitorValuesMap.put(indicatorId, indicatorRecordMap);
        }

        return monitorValuesMap;
    }
}
