package com.persagy.apm.alarmservice.group.objgroup.controller;

import com.persagy.apm.alarmservice.group.objgroup.service.IObjGroupService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.persagy.apm.alarmservice.group.objgroup.model.*;
import com.persagy.apm.alarmservice.group.objgroup.model.vo.*;
import com.persagy.apm.alarmservice.group.objgroup.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import javax.validation.Valid;
import java.util.List;

/**
 * 报警对象分组控制层
 * @author lixing
 * @version V1.0 2021-10-19 14:25:45
 */
@Api(tags = "报警对象分组") 
@Validated
@RestController
@RequestMapping("objGroups")
public class ObjGroupController {
    @Autowired
    private IObjGroupService objGroupService;

	@ApiOperation(value = "创建报警对象分组")
	@PostMapping("/create")
	public CommonResult<ApmResponseUpsertVO> createObjGroup(
	        @Valid @RequestBody AddObjGroupDTO addObjGroupDTO) {
        String id = objGroupService.createObjGroup(addObjGroupDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
	}
	
	@ApiOperation(value = "报警对象分组详情")
    @PostMapping("/get")
    public CommonResult<ObjGroupItemVO> queryObjGroupDetail(
            @Valid @RequestBody QueryObjGroupDetailDTO queryObjGroupDetailDTO) {
        ObjGroup objGroup = objGroupService.queryObjGroupDetail(queryObjGroupDetailDTO.getId());
        ObjGroupItemVO objGroupItemVO = ConvertObjGroupTool.INSTANCE.convert2ResponseItemDTO(objGroup);
        return ResultHelper.single(objGroupItemVO);
    }


    @ApiOperation(value = "更新报警对象分组")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateObjGroup(
            @Valid @RequestBody UpdateObjGroupDTO updateObjGroupDTO) {
        objGroupService.updateObjGroup(updateObjGroupDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateObjGroupDTO.getId()));
    }


    @ApiOperation(value = "删除报警对象分组")
    @PostMapping("/delete")
    public CommonResult deleteObjGroup(
            @RequestBody @Valid DeleteObjGroupDTO deleteObjGroupDTO) {
        objGroupService.deleteObjGroup(deleteObjGroupDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "报警对象分组列表")
    @PostMapping("/query")
    public CommonResult<PageList<ObjGroupListItemVO>> queryObjGroup(
            @Valid @RequestBody QueryObjGroupDTO queryObjGroupDTO) {
        List<ObjGroup> objGroupList = objGroupService.queryObjGroupList(queryObjGroupDTO);
        List<ObjGroupListItemVO> resultList = ConvertObjGroupTool.INSTANCE.convert2List(objGroupList);
        return ResultHelper.multi(resultList);
    }
    
    //@ApiOperation(value = "报警对象分组列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ObjGroupListItemVO>> queryObjGroup(
    //        @Valid @RequestBody PageQueryObjGroupDTO pageQueryObjGroupDTO) {
    //    IPage<ObjGroup> pagedList = objGroupService.pageQueryObjGroup(pageQueryObjGroupDTO);
    //    IPage<ObjGroupListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertObjGroupTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ObjGroup实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}
}

