package com.persagy.apm.alarmservice.group.objgroup.model;

import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;
import com.persagy.apm.alarmservice.group.objgroup.model.vo.*;
import com.persagy.apm.alarmservice.group.objgroup.model.dto.*;

import java.util.List;

/**
 * 报警对象分组(ObjGroup) dto、vo、do转换工具类
 * @author lixing
 * @version V1.0 2021-10-19 14:25:45
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertObjGroupTool {

    ConvertObjGroupTool INSTANCE = Mappers.getMapper(ConvertObjGroupTool.class);
    
    /**
    * do转换为ResponseItemVO
    *
    * @param objGroup do对象
    * @return ResponseItemVO
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    ObjGroupItemVO convert2ResponseItemDTO(ObjGroup objGroup);
    
    /**
    * do转换为ResponseListItemVO
    *
    * @param objGroup do对象
    * @return ResponseListItemVO
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    ObjGroupListItemVO convert2ResponseListItemDTO(ObjGroup objGroup);

    /**
    * do列表转换为ResponseListItemVO列表
    *
    * @param objGroupList do列表
    * @return ResponseListItemVO列表
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    List<ObjGroupListItemVO> convert2List(List<ObjGroup> objGroupList);
    
    /**
    * addDTO转换为do
    *
    * @param addObjGroupDTO addDTO
    * @return 要创建的do对象 
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    ObjGroup convertAddDto2Entity(AddObjGroupDTO addObjGroupDTO);
    
    /**
    * updateDTO转换为实体
    *
    * @param objGroup 更新前的do对象
    * @param updateObjGroupDTO updateDTO
    * @return 更新后的do对象
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    ObjGroup convertUpdateDto2Entity(@MappingTarget ObjGroup objGroup, UpdateObjGroupDTO updateObjGroupDTO);
}

