package com.persagy.apm.alarmservice.group.objgroup.service.impl;

import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.alarmservice.group.objgroup.dao.ObjGroupMapper;
import com.persagy.apm.alarmservice.group.objgroup.service.IObjGroupService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.alarmservice.group.objgroup.model.*;
import com.persagy.apm.alarmservice.group.objgroup.model.dto.*;
import java.util.List;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 报警对象分组(ObjGroup) service层
 *
 * @author lixing
 * @version V1.0 2021-10-19 14:25:45
 */
@Service
public class ObjGroupServiceImpl extends ServiceImpl<ObjGroupMapper, ObjGroup> 
    implements IObjGroupService {
    
    /**
    * 创建报警对象分组
    * @return 报警对象分组主键
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    @Override
    public String createObjGroup(AddObjGroupDTO addObjGroupDTO) {
        ObjGroup objGroup = ConvertObjGroupTool.INSTANCE.convertAddDto2Entity(addObjGroupDTO);
        // 设置默认值
        setDefaultValue(objGroup);
        save(objGroup);
        return objGroup.getId();
    }
    
    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param objGroup 报警对象分组实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ObjGroup objGroup) {
        objGroup.setCreator(PoemsContext.getContext().getUserId());
        // todo 其他默认的属性
        
    }
    
    /**
    * 报警对象分组详情
    * @param id 主键
    * @return 部门do类
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    @Override
    public ObjGroup queryObjGroupDetail(String id) {
        ObjGroup objGroup = getById(id);
        if (objGroup == null) {
            throw new IllegalArgumentException("查看ObjGroup详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return objGroup;
    }
    
    /**
    * 更新报警对象分组
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    @Override
    public void updateObjGroup(UpdateObjGroupDTO updateObjGroupDTO) {
        ObjGroup objGroup = getById(updateObjGroupDTO.getId());
        objGroup = ConvertObjGroupTool.INSTANCE.convertUpdateDto2Entity(objGroup, updateObjGroupDTO);
        objGroup.setModifier(PoemsContext.getContext().getUserId());
        updateById(objGroup);
    }
    
    /**
     * 校验报警对象分组是否可删除
     *
     * @param id 报警对象分组主键
     * @return 报警对象分组do类
     * @author lixing
     * @version V1.0 2021-10-19 14:25:45
     */
    public ObjGroup checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ObjGroup时发生异常，主键为空");
        }

        ObjGroup objGroup = getById(id);

        if (objGroup == null) {
            throw new IllegalArgumentException("删除ObjGroup时发生异常，找不到要删除的数据，id:" + id);
        }

        return objGroup;
    }
    
    /**
    * 删除报警对象分组
    * @param id 主键
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    @Override
    public void deleteObjGroup(String id) {
        // 校验是否可删除
        ObjGroup objGroup = checkDeletable(id);

        objGroup.setValid(ValidEnum.FALSE.getType());
        updateById(objGroup);
    }
    
    /**
    * 查询报警对象分组
    * @return List<ObjGroup>
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    @Override
    public List<ObjGroup> queryObjGroupList(QueryObjGroupDTO queryObjGroupDTO) {
        QueryWrapper<ObjGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjGroup.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ObjGroup.PROP_CREATIONTIME);
        
        if (queryObjGroupDTO != null) {
                                
            if (StringUtils.isNotEmpty(queryObjGroupDTO.getName())) {
                queryWrapper.like(ObjGroup.PROP_NAME, queryObjGroupDTO.getName());
            }
                                        
            if (StringUtils.isNotEmpty(queryObjGroupDTO.getClassCode())) {
                queryWrapper.eq(ObjGroup.PROP_CLASS_CODE, queryObjGroupDTO.getClassCode());
            }
                                        
            if (StringUtils.isNotEmpty(queryObjGroupDTO.getCondition())) {
                queryWrapper.eq(ObjGroup.PROP_CONDITION, queryObjGroupDTO.getCondition());
            }
                                        
            if (StringUtils.isNotEmpty(queryObjGroupDTO.getParentid())) {
                queryWrapper.eq(ObjGroup.PROP_PARENT_ID, queryObjGroupDTO.getParentid());
            }
                                            
        }
        
        return list(queryWrapper);
    }
    
    /**
    * 分页查询报警对象分组
    * @return IPage<ObjGroup>
    * @author lixing
    * @version V1.0 2021-10-19 14:25:45
    */
    @Override
    public IPage<ObjGroup> pageQueryObjGroup(PageQueryObjGroupDTO pageQueryObjGroupDTO) {
        QueryWrapper<ObjGroup> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ObjGroup.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ObjGroup> pageParam = new Page<>(pageQueryObjGroupDTO.getPage(), pageQueryObjGroupDTO.getSize(),true);
        // 排序信息
        if(CollectionUtils.isEmpty(pageQueryObjGroupDTO.getOrders())){
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ObjGroup.PROP_CREATIONTIME);
        }else {
            List<Sort> orders = pageQueryObjGroupDTO.getOrders();
            for(Sort sort: orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE,sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }
                
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryObjGroupDTO.getName())) {
            queryWrapper.like(ObjGroup.PROP_NAME, pageQueryObjGroupDTO.getName());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryObjGroupDTO.getClassCode())) {
            queryWrapper.like(ObjGroup.PROP_CLASS_CODE, pageQueryObjGroupDTO.getClassCode());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryObjGroupDTO.getCondition())) {
            queryWrapper.like(ObjGroup.PROP_CONDITION, pageQueryObjGroupDTO.getCondition());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryObjGroupDTO.getParentid())) {
            queryWrapper.like(ObjGroup.PROP_PARENT_ID, pageQueryObjGroupDTO.getParentid());
        }
                                    
        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }
}
