package com.persagy.apm.alarmservice.project.alarmcondition.model;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.FastjsonTypeHandler;
import com.persagy.apm.alarmservice.group.calculatemethod.model.dto.CalculateMethodDTO;
import com.persagy.apm.common.model.entity.AuditableEntity;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.json.JSONObject;

import java.io.Serializable;
import java.util.List;

/**
 * 报警条件(AlarmCondition)实体类
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:30:38
 */
@EqualsAndHashCode(callSuper = true)
@Data
@ApiModel(description = "报警条件")
@TableName(value = "alarm_condition_v2", autoResultMap = true)
public class AlarmCondition extends AuditableEntity<AlarmCondition> implements Serializable {
    private static final long serialVersionUID = 268657175929005281L;

    @TableField(typeHandler = FastjsonTypeHandler.class)
    @ApiModelProperty("包含的参数")
    private List<String> infoCodes;

    @TableField(typeHandler = FastjsonTypeHandler.class)
    @ApiModelProperty("触发条件-前端展示")
    private List<List<CalculateMethodDTO>> triggerFrontend;

    @ApiModelProperty("触发条件-后端使用")
    private String triggerBackend;

    @ApiModelProperty("触发时长")
    private Integer triggerUphold;

    @ApiModelProperty("报警等级")
    private String alarmLevel;

    @ApiModelProperty("持续多久后恢复")
    private Integer endUphold;

    @ApiModelProperty("触发规则-中文描述")
    @TableField(typeHandler = FastjsonTypeHandler.class)
    private List<String> triggerRules;

    @ApiModelProperty("恢复条件-前端")
    @TableField(typeHandler = FastjsonTypeHandler.class)
    private List<List<CalculateMethodDTO>> endFrontend;

    @ApiModelProperty("恢复条件-后端")
    private String endBackend;

    @ApiModelProperty("恢复规则-中文描述")
    @TableField(typeHandler = FastjsonTypeHandler.class)
    private List<String> endRules;

    @ApiModelProperty("生效时间类型")
    private String effectTimeType;

    @ApiModelProperty("生效时间详情")
    @TableField(typeHandler = FastjsonTypeHandler.class)
    private JSONObject effectTimeDetail;


    public static String PROP_ID = "id";
    public static String PROP_INFO_CODES = "info_codes";
    public static String PROP_TRIGGER_UPHOLD = "trigger_uphold";
    public static String PROP_ALARM_LEVEL = "alarm_level";
    public static String PROP_END_UPHOLD = "end_uphold";
    public static String PROP_TRIGGER_FRONTEND = "trigger_frontend";
    public static String PROP_TRIGGER_BACKEND = "trigger_backend";
    public static String PROP_TRIGGER_RULES = "trigger_rules";
    public static String PROP_END_FRONTEND = "end_frontend";
    public static String PROP_END_BACKEND = "end_backend";
    public static String PROP_END_RULES = "end_rules";
    public static String PROP_EFFECT_TIME_TYPE = "effect_time_type";
    public static String PROP_EFFECT_TIME_DETAIL = "effect_time_detail";

}
