package com.persagy.apm.alarmservice.project.alarmcondition.model;

import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.AddAlarmConditionDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.UpdateAlarmConditionDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.model.vo.AlarmConditionItemVO;
import com.persagy.apm.alarmservice.project.alarmcondition.model.vo.AlarmConditionListItemVO;
import org.mapstruct.*;
import org.mapstruct.MappingTarget;
import org.mapstruct.factory.Mappers;

import java.util.List;

/**
 * 报警条件(AlarmCondition) dto、vo、do转换工具类
 * @author lixing
 * @version V1.0 2021-09-08 22:30:38
 */
@Mapper(nullValueMappingStrategy = NullValueMappingStrategy.RETURN_DEFAULT,
        nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
public interface ConvertAlarmConditionTool {

    ConvertAlarmConditionTool INSTANCE = Mappers.getMapper(ConvertAlarmConditionTool.class);
    
    /**
    * do转换为ResponseItemVO
    *
    * @param alarmCondition do对象
    * @return ResponseItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    @Mappings({
            @Mapping(source = "triggerFrontend", target = "frontend"),
            @Mapping(source = "triggerBackend", target = "backend")
    })
    AlarmConditionItemVO convert2ResponseItemDTO(AlarmCondition alarmCondition);
    
    /**
    * do转换为ResponseListItemVO
    *
    * @param alarmCondition do对象
    * @return ResponseListItemVO
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    @Mappings({
            @Mapping(source = "triggerFrontend", target = "frontend"),
            @Mapping(source = "triggerBackend", target = "backend")
    })
    AlarmConditionListItemVO convert2ResponseListItemDTO(AlarmCondition alarmCondition);

    /**
    * do列表转换为ResponseListItemVO列表
    *
    * @param alarmConditionList do列表
    * @return ResponseListItemVO列表
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    List<AlarmConditionListItemVO> convert2List(List<AlarmCondition> alarmConditionList);
    
    /**
    * addDTO转换为do
    *
    * @param addAlarmConditionDTO addDTO
    * @return 要创建的do对象 
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    AlarmCondition convertAddDto2Entity(AddAlarmConditionDTO addAlarmConditionDTO);
    
    /**
    * updateDTO转换为实体
    *
    * @param alarmCondition 更新前的do对象
    * @param updateAlarmConditionDTO updateDTO
    * @return 更新后的do对象
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    @Mappings({
            @Mapping(source = "updateAlarmConditionDTO.backend", target = "triggerBackend"),
            @Mapping(source = "updateAlarmConditionDTO.frontend", target = "triggerFrontend")
    })
    AlarmCondition convertUpdateDto2Entity(@MappingTarget AlarmCondition alarmCondition, UpdateAlarmConditionDTO updateAlarmConditionDTO);
}

