package com.persagy.apm.alarmservice.project.alarmcondition.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.apm.alarmservice.group.calculatemethod.model.dto.CalculateMethodDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.model.AlarmCondition;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.*;
import com.persagy.apm.alarmservice.project.alarmrule.model.ProjectAlarmRule;
import com.persagy.apm.common.response.CommonResult;

import java.util.List;
import java.util.Map;

/**
 * 报警条件(AlarmCondition) service接口
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:30:38
 */
public interface IAlarmConditionService  {
    
    /**
    * 创建报警条件
    *
    * @param addAlarmConditionDTO addDTO
    * @return 报警条件主键
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    public Map<String, Object> createAlarmCondition(AddAlarmConditionDTO addAlarmConditionDTO);
    
    /**
    * 报警条件详情
    *
    * @param id 主键
    * @return 报警条件do对象
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    public AlarmCondition queryAlarmConditionDetail(String id);
    
    /**
    * 更新报警条件
    *
    * @param updateAlarmConditionDTO updateDTO
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    public void updateAlarmCondition(UpdateAlarmConditionDTO updateAlarmConditionDTO);
    
    /**
    * 删除报警条件
    *
    * @param id 主键
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    public void deleteAlarmCondition(String id);
    
    /**
    * 查询报警条件
    *
    * @param queryAlarmConditionDTO queryDTO
    * @return List<AlarmCondition>
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    public List<AlarmCondition> queryAlarmConditionList(QueryAlarmConditionDTO queryAlarmConditionDTO);
    
    /**
    * 分页查询报警条件
    * 
    * @param pageQueryAlarmConditionDTO pageQueryDTO
    * @return IPage<AlarmCondition>
    * @author lixing
    * @version V1.0 2021-09-08 22:30:38
    */
    public IPage<AlarmCondition> pageQueryAlarmCondition(PageQueryAlarmConditionDTO pageQueryAlarmConditionDTO);

    CommonResult<Boolean> validate(ValidateAlarmConditionDTO formulaString);

    void saveCondition(AlarmCondition alarmCondition);

    Map<String, Object> switchAlarmCondition(AddAlarmConditionDTO addAlarmConditionDTO);

    Map<String, Object> frontend2Backend(List<List<CalculateMethodDTO>> frontend);

    void updateAlarmCondition(Map<String, Object> conditionMap, UpdateAlarmConditionDTO alarmCondition);

    /**
     * 查询所有的报警条件
     *
     * @return 报警条件列表
     * @author lixing
     * @version V1.0 2021/10/22 9:59 上午
     */
    List<AlarmCondition> queryAllAlarmConditions();

    /**
     * 根据设备查询报警条件
     *
     * @param objId 设备id
     * @return 报警条件列表
     * @author lixing
     * @version V1.0 2021/11/3 8:40 下午
     */
    List<AlarmCondition> queryByObj(String objId);

    /**
     * 根据设备和报警类型编码查询报警条件
     *
     * @param objId 设备id
     * @param itemCode 报警类型编码
     * @return 报警条件列表
     * @author lixing
     * @version V1.0 2021/11/3 8:40 下午
     */
    List<AlarmCondition> queryByObjAndItemCode(String objId, String itemCode);

    /**
     * 根据报警规则查询关联的报警条件
     *
     * @param projectAlarmRules 报警规则列表
     * @return 关联的报警条件列表
     * @author lixing
     * @version V1.0 2021/11/3 9:03 下午
     */
    List<AlarmCondition> queryByProjectAlarmRules(List<ProjectAlarmRule> projectAlarmRules);

    /**
     * 根据报警条件id列表查询报警条件
     *
     * @param conditionIdList 报警条件id列表
     * @return 报警条件列表
     * @author lixing
     * @version V1.0 2021/11/3 9:03 下午
     */
    List<AlarmCondition> queryByIds(List<String> conditionIdList);
}
