package com.persagy.apm.alarmservice.project.alarmrule.controller;

import com.persagy.apm.alarmservice.project.alarmcondition.model.AlarmCondition;
import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.alarmservice.project.alarmrule.dao.ProjectAlarmRuleMapper;
import com.persagy.apm.alarmservice.project.alarmrule.model.ConvertProjectAlarmRuleTool;
import com.persagy.apm.alarmservice.project.alarmrule.model.ProjectAlarmRule;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.*;
import com.persagy.apm.alarmservice.project.alarmrule.model.vo.ProjectAlarmRuleItemVO;
import com.persagy.apm.alarmservice.project.alarmrule.model.vo.ProjectAlarmRuleListItemVO;
import com.persagy.apm.alarmservice.project.alarmrule.model.vo.ProjectAlarmRuleWithObjsAndInfoCodes;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.ProjectAlarmRuleObjRel;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.QueryProjectAlarmRuleObjRelDTO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectVO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.common.enums.ResponseCode;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.text.Collator;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 项目报警规则控制层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:46:16
 */
@Api(tags = "项目报警规则")
@Validated
@RestController
@RequestMapping("projectAlarmRules")
public class ProjectAlarmRuleController {
    @Autowired
    private IProjectAlarmRuleService projectAlarmRuleService;
    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;
    @Autowired
    private ProjectAlarmRuleMapper projectAlarmRuleMapper;
    @Autowired
    private IAlarmConditionService alarmConditionService;

    @ApiOperation(value = "查询所有的项目")
    @PostMapping("/projects")
    public CommonResult<List<ProjectVO>> queryProjects() {
        List<ProjectVO> projects = projectAlarmRuleService.getProject();
        return ResultHelper.single(projects);
    }

    @ApiOperation(value = "创建项目报警规则")
    @PostMapping("/create")
    public CommonResult<ApmResponseUpsertVO> createProjectAlarmRule(
            @Valid @RequestBody AddProjectAlarmRuleDTO addProjectAlarmRuleDTO) {
        String id = null;
        try {
            id = projectAlarmRuleService.createProjectAlarmRule(addProjectAlarmRuleDTO);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.single(new ApmResponseUpsertVO(id));
    }

    @ApiOperation(value = "项目报警规则详情")
    @PostMapping("/get")
    @ApiIgnore
    public CommonResult<ProjectAlarmRuleItemVO> queryProjectAlarmRuleDetail(
            @Valid @RequestBody QueryProjectAlarmRuleDetailDTO queryProjectAlarmRuleDetailDTO) {
        ProjectAlarmRule projectAlarmRule = projectAlarmRuleService.queryProjectAlarmRuleDetail(queryProjectAlarmRuleDetailDTO.getId());
        ProjectAlarmRuleItemVO projectAlarmRuleItemVO = ConvertProjectAlarmRuleTool.INSTANCE.convert2ResponseItemDTO(projectAlarmRule);
        return ResultHelper.single(projectAlarmRuleItemVO);
    }


    @ApiOperation(value = "更新项目报警规则")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateProjectAlarmRule(
            @Valid @RequestBody UpdateProjectAlarmRuleDTO updateProjectAlarmRuleDTO) {
        try {
            projectAlarmRuleService.updateProjectAlarmRule(updateProjectAlarmRuleDTO);
        } catch (Exception e) {
            e.printStackTrace();
            return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.single(new ApmResponseUpsertVO(updateProjectAlarmRuleDTO.getId()));
    }


    @ApiOperation(value = "删除项目报警规则")
    @PostMapping("/delete")
    public CommonResult deleteProjectAlarmRule(
            @RequestBody @Valid DeleteProjectAlarmRuleDTO deleteProjectAlarmRuleDTO) {
        try {
            projectAlarmRuleService.deleteProjectAlarmRule(deleteProjectAlarmRuleDTO.getId());
        } catch (Exception e) {
            e.printStackTrace();
            return ResultHelper.failure(ResponseCode.B0001.getCode(), e.getMessage());
        }
        return ResultHelper.success();
    }


    @ApiOperation(value = "项目报警规则列表（不含报警条件）")
    @ApiIgnore
    @PostMapping("/query")
    public CommonResult<PageList<ProjectAlarmRuleListItemVO>> queryProjectAlarmRule(
            @Valid @RequestBody QueryProjectAlarmRuleDTO queryProjectAlarmRuleDTO) {
        List<ProjectAlarmRule> projectAlarmRuleList = projectAlarmRuleService.queryProjectAlarmRuleList(queryProjectAlarmRuleDTO);
        List<ProjectAlarmRuleListItemVO> resultList = ConvertProjectAlarmRuleTool.INSTANCE.convert2List(projectAlarmRuleList);
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "项目报警规则列表, 分页查询")
    @PostMapping("/queryPage")
    public CommonResult<PageList<ProjectAlarmRuleListItemVO>> queryProjectAlarmRule(
            @Valid @RequestBody PageQueryProjectAlarmRuleDTO pageQueryProjectAlarmRuleDTO) {

        int page = pageQueryProjectAlarmRuleDTO.getPage();
        int size = pageQueryProjectAlarmRuleDTO.getSize();
        pageQueryProjectAlarmRuleDTO.setStartIndex((page - 1) * size);
        //        int count = projectAlarmRuleMapper.queryCount(pageQueryProjectAlarmRuleDTO);
        List<ProjectAlarmRuleListItemVO> projectAlarmRuleListItemVOS = projectAlarmRuleMapper.queryList(pageQueryProjectAlarmRuleDTO);
        List<ProjectAlarmRuleListItemVO> result = new ArrayList<>();
        for (ProjectAlarmRuleListItemVO projectAlarmRuleListItemVO : projectAlarmRuleListItemVOS) {
            List<ProjectAlarmRuleObjRel> query = projectAlarmRuleObjRelService.query(projectAlarmRuleListItemVO.getId());
            String equipName = pageQueryProjectAlarmRuleDTO.getEquipName();
            if (StringUtils.isNotBlank(equipName)) {
                for (ProjectAlarmRuleObjRel projectAlarmRuleObjRel : query) {
                    if (StringUtils.isNotBlank(projectAlarmRuleObjRel.getObjName())
                            && projectAlarmRuleObjRel.getObjName().contains(equipName)
                            && projectAlarmRuleObjRel.getOpen() == 1) {
                        projectAlarmRuleListItemVO.setRelListItemList(query);

                    }
                }
                if (null != projectAlarmRuleListItemVO.getRelListItemList())
                    result.add(projectAlarmRuleListItemVO);
            } else {
                projectAlarmRuleListItemVO.setRelListItemList(query);
                result.add(projectAlarmRuleListItemVO);
            }
        }
        // 报警类型名称按照中文排序
        List<ProjectAlarmRuleListItemVO> collect = result.stream().sorted(Comparator.comparing(
                ProjectAlarmRuleListItemVO::getAlarmItemName,
                Collator.getInstance(Locale.CHINA))).collect(Collectors.toList());
        List<ProjectAlarmRuleListItemVO> resultList = projectAlarmRuleService.pageBySubList(collect, size, page);
        return ResultHelper.multi(resultList, result.size());
    }

    @ApiOperation(value = "项目报警规则列表（含关联的监测指标和设备）")
    @PostMapping("/query_with_indicators_objs")
    public CommonResult<PageList<ProjectAlarmRuleWithObjsAndInfoCodes>> queryProjectAlarmRuleWithCondition(
            @Valid @RequestBody QueryProjectAlarmRuleDTO queryProjectAlarmRuleDTO) {
        List<ProjectAlarmRule> projectAlarmRuleList = projectAlarmRuleService.queryProjectAlarmRuleList(queryProjectAlarmRuleDTO);
        List<ProjectAlarmRuleWithObjsAndInfoCodes> resultList = new LinkedList<>();
        for (ProjectAlarmRule projectAlarmRule : projectAlarmRuleList) {
            ProjectAlarmRuleWithObjsAndInfoCodes projectAlarmRuleWithObjsAndInfoCodes =
                    convert2ProjectAlarmRuleWithObjsAndInfoCodes(projectAlarmRule);
            if (projectAlarmRuleWithObjsAndInfoCodes == null) {
                continue;
            }

            resultList.add(projectAlarmRuleWithObjsAndInfoCodes);
        }
        return ResultHelper.multi(resultList);
    }

    /**
     * 项目报警规则对象转换为项目报警规则（含关联的监测指标和设备）
     *
     * @param projectAlarmRule 项目报警规则
     * @return 项目报警规则（含关联的监测指标和设备）
     * @author lixing
     * @version V1.0 2021/10/26 11:14 上午
     */
    private ProjectAlarmRuleWithObjsAndInfoCodes convert2ProjectAlarmRuleWithObjsAndInfoCodes(ProjectAlarmRule projectAlarmRule) {
        ProjectAlarmRuleWithObjsAndInfoCodes projectAlarmRuleWithObjsAndInfoCodes = new ProjectAlarmRuleWithObjsAndInfoCodes();
        projectAlarmRuleWithObjsAndInfoCodes.setId(projectAlarmRule.getId());
        projectAlarmRuleWithObjsAndInfoCodes.setProjectId(projectAlarmRule.getProjectId());
        projectAlarmRuleWithObjsAndInfoCodes.setAlarmItemCode(projectAlarmRule.getAlarmItemCode());
        // 查询报警规则关联的报警条件，从报警条件中取关联的监测指标
        String alarmConditionId = projectAlarmRule.getAlarmConditionId();
        AlarmCondition alarmCondition = alarmConditionService.queryAlarmConditionDetail(alarmConditionId);
        if (alarmCondition == null) {
            return null;
        }
        projectAlarmRuleWithObjsAndInfoCodes.setInfoCodes(alarmCondition.getInfoCodes());
        // 从关联关系表中获取关联的设备
        QueryProjectAlarmRuleObjRelDTO queryProjectAlarmRuleObjRelDTO = new QueryProjectAlarmRuleObjRelDTO();
        queryProjectAlarmRuleObjRelDTO.setProjectAlarmRuleId(projectAlarmRule.getId());
        // 屏蔽状态 1-open、0- 屏蔽
        Integer open = 1;
        queryProjectAlarmRuleObjRelDTO.setOpen(open);
        List<ProjectAlarmRuleObjRel> projectAlarmRuleObjRelList = projectAlarmRuleObjRelService.
                queryProjectAlarmRuleObjRelList(queryProjectAlarmRuleObjRelDTO);
        if (!CollectionUtils.isEmpty(projectAlarmRuleObjRelList)) {
            List<String> objIds = projectAlarmRuleObjRelList.stream().map(
                    ProjectAlarmRuleObjRel::getObjId).collect(Collectors.toList());
            projectAlarmRuleWithObjsAndInfoCodes.setObjIds(objIds);
        }
        return projectAlarmRuleWithObjsAndInfoCodes;
    }
}

