package com.persagy.apm.alarmservice.project.alarmruleobjrel.controller;

import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;

import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.*;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.*;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.List;

/**
 * 项目报警规则与监测对象的关联控制层
 * @author lixing
 * @version V1.0 2021-09-08 22:46:37
 */
@Api(tags = "项目报警规则与监测对象的关联") 
@Validated
@RestController
@RequestMapping("projectAlarmRuleObjRels")
public class ProjectAlarmRuleObjRelController {
    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;
    @Autowired
    private IAlarmConditionService alarmConditionService;
    @Autowired
    private IProjectAlarmRuleService projectAlarmRuleService;

	@ApiOperation(value = "创建项目报警规则与监测对象的关联")
    @ApiIgnore
	@PostMapping("/create")
	public CommonResult<ApmResponseUpsertVO> createProjectAlarmRuleObjRel(
	        @Valid @RequestBody AddProjectAlarmRuleObjRelDTO addProjectAlarmRuleObjRelDTO) {
        String id = projectAlarmRuleObjRelService.createProjectAlarmRuleObjRel(addProjectAlarmRuleObjRelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
	}
	
	@ApiOperation(value = "项目报警规则与监测对象的关联详情")
    @ApiIgnore
    @PostMapping("/get")
    public CommonResult<ProjectAlarmRuleObjRelItemVO> queryProjectAlarmRuleObjRelDetail(
            @Valid @RequestBody QueryProjectAlarmRuleObjRelDetailDTO queryProjectAlarmRuleObjRelDetailDTO) {
        ProjectAlarmRuleObjRel projectAlarmRuleObjRel = projectAlarmRuleObjRelService.queryProjectAlarmRuleObjRelDetail(queryProjectAlarmRuleObjRelDetailDTO.getId());
        ProjectAlarmRuleObjRelItemVO projectAlarmRuleObjRelItemVO = ConvertProjectAlarmRuleObjRelTool.INSTANCE.convert2ResponseItemDTO(projectAlarmRuleObjRel);
        return ResultHelper.single(projectAlarmRuleObjRelItemVO);
    }


    @ApiOperation(value = "更新项目报警规则与监测对象的关联")
    @ApiIgnore
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateProjectAlarmRuleObjRel(
            @Valid @RequestBody UpdateProjectAlarmRuleObjRelDTO updateProjectAlarmRuleObjRelDTO) {
        projectAlarmRuleObjRelService.updateProjectAlarmRuleObjRel(updateProjectAlarmRuleObjRelDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateProjectAlarmRuleObjRelDTO.getId()));
    }


    @ApiOperation(value = "删除项目报警规则与监测对象的关联")
    @ApiIgnore
    @PostMapping("/delete")
    public CommonResult deleteProjectAlarmRuleObjRel(
            @RequestBody @Valid DeleteProjectAlarmRuleObjRelDTO deleteProjectAlarmRuleObjRelDTO) {
        projectAlarmRuleObjRelService.deleteProjectAlarmRuleObjRel(deleteProjectAlarmRuleObjRelDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "项目报警规则与监测对象的关联列表")
    @ApiIgnore
    @PostMapping("/query")
    public CommonResult<PageList<ProjectAlarmRuleObjRelListItemVO>> queryProjectAlarmRuleObjRel(
            @Valid @RequestBody QueryProjectAlarmRuleObjRelDTO queryProjectAlarmRuleObjRelDTO) {
        List<ProjectAlarmRuleObjRel> projectAlarmRuleObjRelList = projectAlarmRuleObjRelService.queryProjectAlarmRuleObjRelList(queryProjectAlarmRuleObjRelDTO);
        List<ProjectAlarmRuleObjRelListItemVO> resultList = ConvertProjectAlarmRuleObjRelTool.INSTANCE.convert2List(projectAlarmRuleObjRelList);
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "监测对象与报警条件的关联列表")
    @ApiIgnore
    @PostMapping("/query_obj_condition_rels")
    public CommonResult<List<ObjConditionRel>> queryObjAlarmConditionRel(
            @Valid @RequestBody QueryObjConditionRelDTO queryObjConditionRelDTO) {
        List<ObjConditionRel> objConditionRels = projectAlarmRuleObjRelService.queryObjAlarmConditionRel(queryObjConditionRelDTO);
        return ResultHelper.single(objConditionRels);
    }
    
    //@ApiOperation(value = "项目报警规则与监测对象的关联列表, 分页查询")
    //@PostMapping("/query")
    //public CommonResult<PageList<ProjectAlarmRuleObjRelListItemVO>> queryProjectAlarmRuleObjRel(
    //        @Valid @RequestBody PageQueryProjectAlarmRuleObjRelDTO pageQueryProjectAlarmRuleObjRelDTO) {
    //    IPage<ProjectAlarmRuleObjRel> pagedList = projectAlarmRuleObjRelService.pageQueryProjectAlarmRuleObjRel(pageQueryProjectAlarmRuleObjRelDTO);
    //    IPage<ProjectAlarmRuleObjRelListItemVO> pagedResultList = pagedList.convert(item -> {
    //        try {
    //           return ConvertProjectAlarmRuleObjRelTool.INSTANCE.convert2ResponseListItemDTO(item);
    //        } catch (Exception e) {
    //            throw new IllegalArgumentException("分页查询，ProjectAlarmRuleObjRel实体类转换为列表对象发生异常", e);
    //        }
    //    });
    //    return ResultHelper.multi(pagedResultList);
    //}

    @ApiOperation(value = "修改屏蔽管理")
    @PostMapping("/shieldManage")
    public CommonResult<ApmResponseUpsertVO> shieldManage(
            @Valid @RequestBody ShieldManageListDTO shieldManageListDTO) throws Exception {
        projectAlarmRuleObjRelService.updateBatch(shieldManageListDTO);
        return ResultHelper.success();
    }

    @ApiOperation(value = "项目报警规则与监测对象的关联列表")
    @PostMapping("/queryShieldManage")
    public CommonResult<PageList<ProjectAlarmRuleObjRelListItemVO>> queryShieldManage(
            @Valid @RequestBody QueryShieldManageDTO queryShieldManageDTO) {
        List<ProjectAlarmRuleObjRel> projectAlarmRuleObjRelList = projectAlarmRuleObjRelService.queryShieldManage(queryShieldManageDTO);
        List<ProjectAlarmRuleObjRelListItemVO> resultList = ConvertProjectAlarmRuleObjRelTool.INSTANCE.convert2List(projectAlarmRuleObjRelList);
        return ResultHelper.multi(resultList);
    }

    @ApiOperation(value = "查询项目报警对象")
    @PostMapping("/queryRuleObjRel")
    public CommonResult<PageList<ProjectAlarmRuleObjRelListItemVO>> queryRuleObjRel(
            @Valid @RequestBody QueryRuleObjRelDTO queryRuleObjRelDTO) {
        List<ProjectAlarmRuleObjRelListItemVO> projectAlarmRuleObjRelVOS = projectAlarmRuleObjRelService.queryRuleObjRel(queryRuleObjRelDTO);
        return ResultHelper.multi(projectAlarmRuleObjRelVOS);
    }
}

