package com.persagy.apm.alarmservice.project.alarmruleobjrel.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.google.common.base.Function;
import com.google.common.base.Predicate;
import com.persagy.apm.alarmservice.common.jms.JmsSender;
import com.persagy.apm.alarmservice.group.alarmrule.model.GroupAlarmRule;
import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.service.GroupManagementService;
import com.persagy.apm.alarmservice.project.alarmrule.model.ProjectAlarmRule;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.QueryProjectAlarmRuleDTO;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.dao.ProjectAlarmRuleObjRelMapper;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.ConvertProjectAlarmRuleObjRelTool;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.ProjectAlarmRuleObjRel;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.*;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ObjConditionRel;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectAlarmRuleObjRelListItemVO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectVO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.BatchDeleteAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.DeleteAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.QueryAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmConfigItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmConfigServiceImpl;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

/**
 * 项目报警规则与监测对象的关联(ProjectAlarmRuleObjRel) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:46:37
 */
@Service
public class ProjectAlarmRuleObjRelServiceImpl extends ServiceImpl<ProjectAlarmRuleObjRelMapper, ProjectAlarmRuleObjRel>
        implements IProjectAlarmRuleObjRelService {

    @Autowired
    private IProjectAlarmRuleService projectAlarmRuleService;

    @Autowired
    AlarmConfigServiceImpl alarmConfigService;

    @Autowired
    GroupManagementService groupManagementService;
    @Autowired
    private ProjectAlarmRuleObjRelMapper projectAlarmRuleObjRelMapper;
    @Autowired
    JmsSender jmsSender;

    /**
     * 创建项目报警规则与监测对象的关联
     *
     * @return 项目报警规则与监测对象的关联主键
     * @author lixing
     * @version V1.0 2021-09-08 22:46:37
     */
    @Override
    public String createProjectAlarmRuleObjRel(AddProjectAlarmRuleObjRelDTO addProjectAlarmRuleObjRelDTO) {
        ProjectAlarmRuleObjRel projectAlarmRuleObjRel = ConvertProjectAlarmRuleObjRelTool.INSTANCE.convertAddDto2Entity(addProjectAlarmRuleObjRelDTO);
        // 设置默认值
        setDefaultValue(projectAlarmRuleObjRel);
        save(projectAlarmRuleObjRel);
        return projectAlarmRuleObjRel.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param projectAlarmRuleObjRel 项目报警规则与监测对象的关联实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ProjectAlarmRuleObjRel projectAlarmRuleObjRel) {
        projectAlarmRuleObjRel.setCreator(AppContext.getContext().getAccountId());
    }

    /**
     * 项目报警规则与监测对象的关联详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:37
     */
    @Override
    public ProjectAlarmRuleObjRel queryProjectAlarmRuleObjRelDetail(String id) {
        ProjectAlarmRuleObjRel projectAlarmRuleObjRel = getById(id);
        if (projectAlarmRuleObjRel == null) {
            throw new IllegalArgumentException("查看ProjectAlarmRuleObjRel详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return projectAlarmRuleObjRel;
    }

    /**
     * 更新项目报警规则与监测对象的关联
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:46:37
     */
    @Override
    public void updateProjectAlarmRuleObjRel(UpdateProjectAlarmRuleObjRelDTO updateProjectAlarmRuleObjRelDTO) {
        ProjectAlarmRuleObjRel projectAlarmRuleObjRel = getById(updateProjectAlarmRuleObjRelDTO.getId());
        projectAlarmRuleObjRel = ConvertProjectAlarmRuleObjRelTool.INSTANCE.convertUpdateDto2Entity(projectAlarmRuleObjRel, updateProjectAlarmRuleObjRelDTO);
        projectAlarmRuleObjRel.setModifier(AppContext.getContext().getAccountId());
        updateById(projectAlarmRuleObjRel);
    }

    /**
     * 校验项目报警规则与监测对象的关联是否可删除
     *
     * @param id 项目报警规则与监测对象的关联主键
     * @return 项目报警规则与监测对象的关联do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:37
     */
    public ProjectAlarmRuleObjRel checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ProjectAlarmRuleObjRel时发生异常，主键为空");
        }

        ProjectAlarmRuleObjRel projectAlarmRuleObjRel = getById(id);

        if (projectAlarmRuleObjRel == null) {
            throw new IllegalArgumentException("删除ProjectAlarmRuleObjRel时发生异常，找不到要删除的数据，id:" + id);
        }

        return projectAlarmRuleObjRel;
    }

    /**
     * 删除项目报警规则与监测对象的关联
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:46:37
     */
    @Override
    public void deleteProjectAlarmRuleObjRel(String id) {
        // 校验是否可删除
        ProjectAlarmRuleObjRel projectAlarmRuleObjRel = checkDeletable(id);

        projectAlarmRuleObjRel.setValid(ValidEnum.FALSE.getType());
        updateById(projectAlarmRuleObjRel);
    }

    /**
     * 查询项目报警规则与监测对象的关联
     *
     * @return List<ProjectAlarmRuleObjRel>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:37
     */
    @Override
    public List<ProjectAlarmRuleObjRel> queryProjectAlarmRuleObjRelList(QueryProjectAlarmRuleObjRelDTO queryProjectAlarmRuleObjRelDTO) {
        QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectAlarmRuleObjRel.PROP_CREATIONTIME);

        if (queryProjectAlarmRuleObjRelDTO != null) {

            if (StringUtils.isNotEmpty(queryProjectAlarmRuleObjRelDTO.getProjectAlarmRuleId())) {
                queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, queryProjectAlarmRuleObjRelDTO.getProjectAlarmRuleId());
            }

            if (StringUtils.isNotEmpty(queryProjectAlarmRuleObjRelDTO.getEquipId())) {
                queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_OBJ_ID, queryProjectAlarmRuleObjRelDTO.getEquipId());
            }

            if (queryProjectAlarmRuleObjRelDTO.getOpen() != null) {
                queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_OPEN, queryProjectAlarmRuleObjRelDTO.getOpen());
            }

            if (StringUtils.isNotEmpty(queryProjectAlarmRuleObjRelDTO.getProjectId())) {
                queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ID, queryProjectAlarmRuleObjRelDTO.getProjectId());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询项目报警规则与监测对象的关联
     *
     * @return IPage<ProjectAlarmRuleObjRel>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:37
     */
    @Override
    public IPage<ProjectAlarmRuleObjRel> pageQueryProjectAlarmRuleObjRel(PageQueryProjectAlarmRuleObjRelDTO pageQueryProjectAlarmRuleObjRelDTO) {
        QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ProjectAlarmRuleObjRel> pageParam = new Page<>(pageQueryProjectAlarmRuleObjRelDTO.getPage(), pageQueryProjectAlarmRuleObjRelDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryProjectAlarmRuleObjRelDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, ProjectAlarmRuleObjRel.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryProjectAlarmRuleObjRelDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleObjRelDTO.getProjectAlarmRuleId())) {
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, pageQueryProjectAlarmRuleObjRelDTO.getProjectAlarmRuleId());
        }

        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleObjRelDTO.getEquipId())) {
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_OBJ_ID, pageQueryProjectAlarmRuleObjRelDTO.getEquipId());
        }

        if (pageQueryProjectAlarmRuleObjRelDTO.getOpen() != null) {
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_OPEN, pageQueryProjectAlarmRuleObjRelDTO.getOpen());
        }

        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleObjRelDTO.getProjectId())) {
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ID, pageQueryProjectAlarmRuleObjRelDTO.getProjectId());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public List<ProjectAlarmRuleObjRel> queryRelationListByProjectIds(List<String> projectIds) {
        QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.in(ProjectAlarmRuleObjRel.PROP_PROJECT_ID, projectIds);
        return list(queryWrapper);
    }

    @Override
    public void deleteRelation(String ruleId) {
        QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, ruleId);
        ProjectAlarmRuleObjRel projectAlarmRuleObjRel = new ProjectAlarmRuleObjRel();
        projectAlarmRuleObjRel.setValid(ValidEnum.FALSE.getType());
        jmsSender.sendRemoveObjConditionRelListMsg(ruleId);
        update(projectAlarmRuleObjRel, queryWrapper);
    }

    @Override
    public void createBatchRelationAndSendMsg(List<AddProjectAlarmRuleObjRelDTO> addRelationList) {
        List<ProjectAlarmRuleObjRel> relationList = new ArrayList<>();
        addRelationList.forEach(ruleObjRel -> {
            ProjectAlarmRuleObjRel projectAlarmRuleObjRel = ConvertProjectAlarmRuleObjRelTool.INSTANCE.convertAddDto2Entity(ruleObjRel);
            setDefaultValue(projectAlarmRuleObjRel);
            relationList.add(projectAlarmRuleObjRel);
        });
        saveBatchAndSendMsg(relationList);
    }

    @Override
    public List<ProjectAlarmRuleObjRel> createBatchRelation(List<AddProjectAlarmRuleObjRelDTO> addRelationList) {
        List<ProjectAlarmRuleObjRel> relationList = new ArrayList<>();
        addRelationList.forEach(ruleObjRel -> {
            ProjectAlarmRuleObjRel projectAlarmRuleObjRel = ConvertProjectAlarmRuleObjRelTool.INSTANCE.convertAddDto2Entity(ruleObjRel);
            setDefaultValue(projectAlarmRuleObjRel);
            relationList.add(projectAlarmRuleObjRel);
        });
        saveBatch(relationList);
        return relationList;
    }

    @Override
    public void updateBatch(ShieldManageListDTO shieldManageListDTO) throws Exception {
        List<ShieldManageDTO> shieldManageDTOList = shieldManageListDTO.getShieldManageDTOList();
        String projectAlarmRuleId = shieldManageListDTO.getProjectAlarmRuleId();
        String projectId = shieldManageListDTO.getProjectId();
        // 更新前，发送移除老的关联关系的消息
        jmsSender.sendRemoveObjConditionRelListMsg(projectAlarmRuleId);
        projectAlarmRuleObjRelMapper.updateStatus(shieldManageDTOList, projectAlarmRuleId, projectId);
        // 更新后，发送增加新的关联关系的消息
        jmsSender.sendNewObjConditionRelListMsg(projectAlarmRuleId);

//        //删除设备报警条件 begin
//        BatchDeleteAlarmConfigDTO batchDeleteAlarmConfigDTO = new BatchDeleteAlarmConfigDTO();
//        List<DeleteAlarmConfigDTO> alarmConfigs = new ArrayList<>();
//        String condition = shieldManageListDTO.getCondition();
//        String classCode = shieldManageListDTO.getClassCode();
//        //查询所有设备
//        List<EquipItem> equips = this.getAllEquip(classCode, condition);
//        List<String> objIds = equips.stream().map(EquipItem::getId).collect(Collectors.toList());
//        QueryAlarmConfigDTO queryAlarmConfigDTO = new QueryAlarmConfigDTO();
//        queryAlarmConfigDTO.setObjId(objIds);
//        DmpResult<List<AlarmConfigItem>> query = alarmConfigService.query(queryAlarmConfigDTO);
//        //所有设备条件
//        List<AlarmConfigItem> data = query.getData();
//        //组织删除参数
//        if (data.size() > 0) {
//            data.forEach(alarmConfigItem -> {
//                DeleteAlarmConfigDTO deleteAlarmConfigDTO = new DeleteAlarmConfigDTO();
//                deleteAlarmConfigDTO.setId(alarmConfigItem.getId());
//                alarmConfigs.add(deleteAlarmConfigDTO);
//            });
//            batchDeleteAlarmConfigDTO.setAlarmConfigs(alarmConfigs);
//            alarmConfigService.batchDelete(batchDeleteAlarmConfigDTO);
//        }
//        //删除设备报警条件 end
    }

    public List<EquipItem> getAllEquip(String classCode, String condition) {
        //查询所有项目
        List<ProjectVO> projects = projectAlarmRuleService.getProject();
        List<EquipItem> records = new ArrayList<>();
        for (ProjectVO project : projects) {
            QueryEquipDTO queryEquipDTO = new QueryEquipDTO();
            queryEquipDTO.setClassCode(classCode);
            queryEquipDTO.setCondition(condition);
            List<EquipItem> record = groupManagementService.getEquipList(queryEquipDTO);
            records.addAll(record);
        }
        return records;
    }

    @Override
    public List<ProjectAlarmRuleObjRel> queryShieldManage(QueryShieldManageDTO queryShieldManageDTO) {
        QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectAlarmRuleObjRel.PROP_CREATIONTIME);

        if (queryShieldManageDTO != null) {

            if (StringUtils.isNotEmpty(queryShieldManageDTO.getProjectAlarmRuleId())) {
                queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, queryShieldManageDTO.getProjectAlarmRuleId());
            }

            if (StringUtils.isNotEmpty(queryShieldManageDTO.getProjectId())) {
                queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ID, queryShieldManageDTO.getProjectId());
            }

        }

        return list(queryWrapper);
    }

    @Override
    public List<ProjectAlarmRuleObjRelListItemVO> queryRuleObjRel(QueryRuleObjRelDTO queryRuleObjRelDTO) {
        List<ProjectAlarmRuleObjRel> resultList = new ArrayList<>();
        if (StringUtils.isNotBlank(queryRuleObjRelDTO.getProjectAlarmRuleId())) {
            //1、查询项目报警下的未屏蔽的报警对象
            QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, queryRuleObjRelDTO.getProjectAlarmRuleId());
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_VALID, ValidEnum.TRUE.getType());
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_OPEN, 1);
            List<ProjectAlarmRuleObjRel> list = list(queryWrapper);
            list.stream().map(ProjectAlarmRuleObjRel::getObjId).distinct().collect(Collectors.toList());
            resultList.addAll(list);
        }
        //2、查询未绑定的报警对象（未屏蔽、未关联）
        //查询所有设备（报警对象）
        String projectId = queryRuleObjRelDTO.getProjectId();
        QueryEquipDTO queryEquipDTO = new QueryEquipDTO();
        queryEquipDTO.setProjectId(projectId);
        queryEquipDTO.setClassCode(queryRuleObjRelDTO.getClassCode());
        queryEquipDTO.setCondition(queryRuleObjRelDTO.getCondition());
        List<EquipItem> records = groupManagementService.getEquipList(queryEquipDTO);
        //查询该报警类型下所有（可能有多个相同报警类型的项目报警）已绑定的报警对象
        List<String> projectAlarmRuleIdList = projectAlarmRuleService.queryAlarmRule(queryRuleObjRelDTO);
        Map<String, ProjectAlarmRuleObjRel> partitionMap = new HashMap<>();
        if (projectAlarmRuleIdList.size() > 0) {
            QueryWrapper<ProjectAlarmRuleObjRel> wrapper = new QueryWrapper<>();
            wrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ID, projectId);
            wrapper.in(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, projectAlarmRuleIdList);
            wrapper.eq(ProjectAlarmRuleObjRel.PROP_VALID, ValidEnum.TRUE.getType());
            List<ProjectAlarmRuleObjRel> listRel = list(wrapper);
            partitionMap = listRel.stream().collect(Collectors.toMap(ProjectAlarmRuleObjRel::getObjId, equip -> equip, (k1, k2) -> k1));
        }
        //筛选未绑定的报警对象
        for (EquipItem record : records) {
            String id = record.getId();
            if (!partitionMap.containsKey(id)) {
                ProjectAlarmRuleObjRel projectAlarmRuleObjRelVO = new ProjectAlarmRuleObjRel();
                projectAlarmRuleObjRelVO.setObjId(record.getId());
                projectAlarmRuleObjRelVO.setObjName(record.getLocalName());
                projectAlarmRuleObjRelVO.setOpen(1);
                projectAlarmRuleObjRelVO.setProjectId(projectId);
                resultList.add(projectAlarmRuleObjRelVO);
            }
        }
        return ConvertProjectAlarmRuleObjRelTool.INSTANCE.convert2List(resultList);
    }

    private static <T> Predicate<T> distinctByKey(Function<? super T, Object> keyExtractor) {
        Map<Object, Boolean> seen = new ConcurrentHashMap<>();
        return t -> seen.putIfAbsent(keyExtractor.apply(t), Boolean.TRUE) == null;
    }


    @Override
    public List<ProjectAlarmRuleObjRel> query(String id) {
        QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, id);
        queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_VALID, ValidEnum.TRUE.getType());
        return list(queryWrapper);
    }

    @Override
    public List<ProjectAlarmRuleObjRel> queryObjRelList(List<String> projectAlarmRuleIdList) {
        List<ProjectAlarmRuleObjRel> list = new ArrayList<>();
        for (String projectAlarmRuleId : projectAlarmRuleIdList) {
            QueryWrapper<ProjectAlarmRuleObjRel> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_PROJECT_ALARM_RULE_ID, projectAlarmRuleId);
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_VALID, ValidEnum.TRUE.getType());
            queryWrapper.eq(ProjectAlarmRuleObjRel.PROP_OPEN, 1);
            list.addAll(list(queryWrapper));

        }
        list.stream().map(ProjectAlarmRuleObjRel::getObjId).distinct().collect(Collectors.toList());
        return list;
    }

    @Override
    public void saveObjRuleRels(List<ProjectAlarmRuleObjRel> relList) {
        saveBatchAndSendMsg(relList);
    }

    @Override
    public List<ObjConditionRel> queryObjAlarmConditionRel(QueryObjConditionRelDTO queryObjConditionRelDTO) {
        return projectAlarmRuleObjRelMapper.queryObjAlarmConditionRel(queryObjConditionRelDTO.getProjectId());
    }

    /**
     * 批量保存并发送mq消息
     *
     * @param relList 设备与项目报警规则关联关系
     * @author lixing
     * @version V1.0 2021/10/26 3:21 下午
     */
    private void saveBatchAndSendMsg(List<ProjectAlarmRuleObjRel> relList) {
        saveBatch(relList);
        jmsSender.sendNewObjConditionRelListMsg(relList);
    }

    @Override
    public List<String> queryRuleIdsByObj(String objId) {
        QueryProjectAlarmRuleObjRelDTO queryProjectAlarmRuleObjRelDTO = new QueryProjectAlarmRuleObjRelDTO();
        queryProjectAlarmRuleObjRelDTO.setEquipId(objId);
        // 屏蔽状态 1-open、0- 屏蔽
        queryProjectAlarmRuleObjRelDTO.setOpen(1);
        List<ProjectAlarmRuleObjRel> projectAlarmRuleObjRels = queryProjectAlarmRuleObjRelList(queryProjectAlarmRuleObjRelDTO);
        if (CollectionUtils.isEmpty(projectAlarmRuleObjRels)) {
            return new ArrayList<>();
        }
        return projectAlarmRuleObjRels.stream().map(
                ProjectAlarmRuleObjRel::getProjectAlarmRuleId).collect(Collectors.toList());
    }
}
